import sys
from multiprocessing import Pool

import pytest

import enchant


def check_words(words):
    d = enchant.Dict("en-US")
    for word in words:
        d.check(word)
    return True


@pytest.mark.skipif(
    sys.implementation.name == "pypy" and sys.platform == "win32",
    reason="hangs for an unknown reason",
)
def test_can_use_multiprocessing():
    words = ["hello" for i in range(1000)]
    input = [words for i in range(1000)]
    pool = Pool(10)
    assert all(pool.imap_unordered(check_words, input))



from enchant.utils import trim_suggestions


def test_trim_suggestions():
    word = "gud"
    suggs = ["good", "god", "bad+"]
    assert trim_suggestions(word, suggs, 40) == ["god", "good", "bad+"]
    assert trim_suggestions(word, suggs, 4) == ["god", "good", "bad+"]
    assert trim_suggestions(word, suggs, 3) == ["god", "good", "bad+"]
    assert trim_suggestions(word, suggs, 2) == ["god", "good"]
    assert trim_suggestions(word, suggs, 1) == ["god"]
    assert trim_suggestions(word, suggs, 0) == []



import enchant


def test_get_user_config_dir():
    """
    Scenario:
      Either broker.get_user_config_dir() works
      (enchant >= 2.0), or it throws AttributeError
      (enchant < 2.0)

    """
    try:
        user_dir = enchant.get_user_config_dir()
        assert user_dir
    except AttributeError:
        assert True



"""Test cases for the proper functioning of Dict objects.
These tests assume that there is at least one working provider
with a dictionary for the "en_US" language.
"""

import pickle

import pytest

import enchant
from enchant import Dict, DictNotFoundError, Error, dict_exists
from enchant.utils import get_default_language


@pytest.fixture
def en_us_dict():
    res = Dict("en_US")
    yield res
    del res


def test_has_en_us():
    """Test that the en_US language is available through default broker."""
    assert dict_exists("en_US")


def test_check(en_us_dict):
    """Test that check() works on some common words."""
    assert en_us_dict.check("hello")
    assert en_us_dict.check("test")
    assert not en_us_dict.check("helo")
    assert not en_us_dict.check("testt")
    with pytest.raises(ValueError):
        en_us_dict.check("")


def test_broker(en_us_dict):
    """Test that the dict's broker is set correctly."""
    assert en_us_dict._broker is enchant._broker


def test_tag(en_us_dict):
    """Test that the dict's tag is set correctly."""
    assert en_us_dict.tag == "en_US"


def test_suggest(en_us_dict):
    """Test that suggest() gets simple suggestions right."""
    assert en_us_dict.check("hello")
    assert "hello" in en_us_dict.suggest("helo")
    with pytest.raises(ValueError):
        en_us_dict.suggest("")


def test_suggest_hang_1(en_us_dict):
    """Test whether suggest() hangs on some inputs (Bug #1404196)"""
    assert en_us_dict.suggest("Thiis")
    assert len(en_us_dict.suggest("Thiiis")) >= 0
    assert len(en_us_dict.suggest("Thiiiis")) >= 0


def test_unicode1(en_us_dict):
    """Test checking/suggesting for unicode strings"""
    # TODO: find something that actually returns suggestions
    us1 = r"he\u2149lo"
    assert type(us1) is str
    assert not en_us_dict.check(us1)
    for s in en_us_dict.suggest(us1):
        assert type(s) is str


def test_session(en_us_dict):
    """Test that adding words to the session works as required."""
    assert not en_us_dict.check("Lozz")
    assert not en_us_dict.is_added("Lozz")

    en_us_dict.add_to_session("Lozz")
    assert en_us_dict.is_added("Lozz")
    assert en_us_dict.check("Lozz")

    en_us_dict.remove_from_session("Lozz")
    assert not en_us_dict.check("Lozz")
    assert not en_us_dict.is_added("Lozz")

    en_us_dict.remove_from_session("hello")
    assert not en_us_dict.check("hello")
    assert en_us_dict.is_removed("hello")

    # TODO: fixture please
    en_us_dict.add_to_session("hello")


def test_add_remove(en_us_dict):
    """Test adding/removing from default user dictionary."""
    nonsense = "kxhjsddsi"
    assert not en_us_dict.check(nonsense)
    en_us_dict.add(nonsense)
    assert en_us_dict.is_added(nonsense)
    assert en_us_dict.check(nonsense)

    en_us_dict.remove(nonsense)
    assert not en_us_dict.is_added(nonsense)
    assert not en_us_dict.check(nonsense)
    en_us_dict.remove("pineapple")

    assert not en_us_dict.check("pineapple")
    assert en_us_dict.is_removed("pineapple")
    assert not en_us_dict.is_added("pineapple")
    en_us_dict.add("pineapple")
    assert en_us_dict.check("pineapple")


def test_default_lang(en_us_dict):
    """Test behaviour of default language selection."""
    def_lang = get_default_language()
    if def_lang is None:
        # If no default language, shouldn't work
        with pytest.raises(Error):
            Dict()
    else:
        # If there is a default language, should use it
        # Of course, no need for the dict to actually exist
        try:
            d = Dict()
            assert d.tag == def_lang
        except DictNotFoundError:
            pass


def test_pickling(en_us_dict):
    """Test that pickling doesn't corrupt internal state."""
    d1 = Dict("en_US")
    assert d1.check("hello")
    d2 = pickle.loads(pickle.dumps(d1))
    assert d1.check("hello")
    assert d2.check("hello")
    d1._free()
    assert d2.check("hello")



# pyenchant
#
# Copyright (C) 2004-2009, Ryan Kelly
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
#
# In addition, as a special exception, you are
# given permission to link the code of this program with
# non-LGPL Spelling Provider libraries (eg: a MSFT Office
# spell checker backend) and distribute linked combinations including
# the two.  You must obey the GNU Lesser General Public License in all
# respects for all of the code used other than said providers.  If you modify
# this file, you may extend this exception to your version of the
# file, but you are not obligated to do so.  If you do not wish to
# do so, delete this exception statement from your version.

import array

import pytest

import enchant
import enchant.tokenize
from enchant.checker import SpellChecker
from enchant.errors import DefaultLanguageNotFoundError
from enchant.utils import get_default_language


def test_basic():
    """Test a basic run of the SpellChecker class."""
    text = """This is sme text with a few speling erors in it. Its gret
    for checking wheather things are working proprly with the SpellChecker
    class. Not gret for much elss though."""
    chkr = SpellChecker("en_US", text=text)
    for n, err in enumerate(chkr):
        if n == 0:
            # Fix up "sme" -> "some" properly
            assert err.word == "sme"
            assert err.wordpos == 8
            assert "some" in err.suggest()
            err.replace("some")
        if n == 1:
            # Ignore "speling"
            assert err.word == "speling"
        if n == 2:
            # Check context around "erors", and replace
            assert err.word == "erors"
            assert err.leading_context(5) == "ling "
            assert err.trailing_context(5) == " in i"
            err.replace("errors")
        if n == 3:
            # Replace-all on gret as it appears twice
            assert err.word == "gret"
            err.replace_always("great")
        if n == 4:
            # First encounter with "wheather", move offset back
            assert err.word == "wheather"
            err.set_offset(-1 * len(err.word))
        if n == 5:
            # Second encounter, fix up "wheather'
            assert err.word == "wheather"
            err.replace("whether")
        if n == 6:
            # Just replace "proprly", but also add an ignore
            # for "SpellChecker"
            assert err.word == "proprly"
            err.replace("properly")
            err.ignore_always("SpellChecker")
        if n == 7:
            # The second "gret" should have been replaced
            # So it's now on "elss"
            assert err.word == "elss"
            err.replace("else")
        if n > 7:
            pytest.fail("Extraneous spelling errors were found")
    text2 = """This is some text with a few speling errors in it. Its great
    for checking whether things are working properly with the SpellChecker
    class. Not great for much else though."""
    assert chkr.get_text() == text2


def test_filters():
    """Test SpellChecker with the 'filters' argument."""
    text = """I contain WikiWords that ShouldBe skipped by the filters"""
    chkr = SpellChecker("en_US", text=text, filters=[enchant.tokenize.WikiWordFilter])
    for err in chkr:
        # There are no errors once the WikiWords are skipped
        pytest.fail("Extraneous spelling errors were found")
    assert chkr.get_text() == text


def test_chunkers():
    """Test SpellChecker with the 'chunkers' argument."""
    text = """I contain <html a=xjvf>tags</html> that should be skipped"""
    chkr = SpellChecker("en_US", text=text, chunkers=[enchant.tokenize.HTMLChunker])
    for err in chkr:
        # There are no errors when the <html> tag is skipped
        pytest.fail("Extraneous spelling errors were found")
    assert chkr.get_text() == text


class TestChunkersAndFilters:
    """Test SpellChecker with the 'chunkers' and 'filters' arguments."""

    text = """I contain <xjvf a=xjvf>tags</xjvf> that should be skipped
              along with a <a href='http://example.com/">link to
              http://example.com/</a> that should also be skipped"""

    def test_chunkers_and_filters(self) -> None:
        # There are no errors when things are correctly skipped
        chkr = SpellChecker(
            "en_US",
            text=self.text,
            filters=[enchant.tokenize.URLFilter],
            chunkers=[enchant.tokenize.HTMLChunker],
        )
        for err in chkr:
            pytest.fail("Extraneous spelling errors were found")
        assert chkr.get_text() == self.text

    def test_filter_only(self) -> None:
        # The "html" is an error when not using HTMLChunker
        chkr = SpellChecker(
            "en_US",
            text=self.text,
            filters=[enchant.tokenize.URLFilter],
        )
        for err in chkr:
            assert err.word == "xjvf"
            break
        assert chkr.get_text() == self.text

    def test_chunkter_only(self) -> None:
        # The "http" from the URL is an error when not using URLFilter
        chkr = SpellChecker(
            "en_US",
            text=self.text,
            chunkers=[enchant.tokenize.HTMLChunker],
        )
        for err in chkr:
            assert err.word == "http"
            break
        assert chkr.get_text() == self.text


def test_unicode():
    """Test SpellChecker with a unicode string."""
    text = """I am a unicode strng with unicode erors."""
    chkr = SpellChecker("en_US", text)
    for n, err in enumerate(chkr):
        if n == 0:
            assert err.word == "unicode"
            assert err.wordpos == 7
            chkr.ignore_always()
        if n == 1:
            assert err.word == "strng"
            chkr.replace_always("string")
            assert chkr._replace_words["strng"] == "string"
        if n == 2:
            assert err.word == "erors"
            chkr.replace("erros")
            chkr.set_offset(-6)
        if n == 3:
            assert err.word == "erros"
            chkr.replace("errors")
    assert n == 3
    assert chkr.get_text() == "I am a unicode string with unicode errors."


def test_chararray():
    """Test SpellChecker with a character array as input."""
    atype = "u"
    text = "I wll be stord in an aray"
    txtarr = array.array(atype, text)
    chkr = SpellChecker("en_US", txtarr)
    for n, err in enumerate(chkr):
        if n == 0:
            assert err.word == "wll"
            assert err.word.__class__ == str
        if n == 1:
            assert err.word == "stord"
            txtarr[err.wordpos : err.wordpos + len(err.word)] = array.array(
                atype, "stored"
            )
            chkr.set_offset(-1 * len(err.word))
        if n == 2:
            assert err.word == "aray"
            chkr.replace("array")
    assert n == 2
    assert txtarr.tounicode() == "I wll be stored in an array"


def test_pwl():
    """Test checker loop with PWL."""
    from enchant import DictWithPWL

    d = DictWithPWL("en_US", None, None)
    txt = "I am sme text to be cheked with personal list of cheked words"
    chkr = SpellChecker(d, txt)
    for n, err in enumerate(chkr):
        if n == 0:
            assert err.word == "sme"
        if n == 1:
            assert err.word == "cheked"
            chkr.add()
    assert n == 1


def test_bug2785373():
    """Testcases for bug #2785373."""
    c = SpellChecker(enchant.Dict("en_US"), "")
    c.set_text("So, one dey when I wes 17, I left.")
    for err in c:
        pass
    c = SpellChecker(enchant.Dict("en_US"), "")
    c.set_text("So, one dey when I wes 17, I left.")
    for err in c:
        pass


def test_default_language():
    # Two cases: either SpellChecker() without argument works
    # and its lang is the default language, or
    # it does not and we get a DefaultLanguageNotFoundError
    try:
        checker = SpellChecker()
    except DefaultLanguageNotFoundError:
        # At this point, caught_err must be DefaultLanguageNotFoundError, so
        # we're done testing
        return

    assert checker.lang == get_default_language()


def test_replace_with_shorter_string():
    """Testcase for replacing with a shorter string (bug #10)"""
    text = ". I Bezwaar tegen verguning."
    chkr = SpellChecker("en_US", text)
    for i, err in enumerate(chkr):
        err.replace("SPAM")
        assert i < 3
    assert chkr.get_text() == ". I SPAM SPAM SPAM."


def test_replace_with_empty_string():
    """Testcase for replacing with an empty string (bug #10)"""
    text = ". I Bezwaar tegen verguning."
    chkr = SpellChecker("en_US", text)
    for i, err in enumerate(chkr):
        err.replace("")
        assert i < 3
    assert chkr.get_text() == ". I   ."



import pytest

from enchant import Broker


@pytest.fixture
def broker():
    res = Broker()
    yield res
    del res


def test_all_langs_are_available(broker):
    """Test whether all advertised languages are in fact available."""
    for lang in broker.list_languages():
        if not broker.dict_exists(lang):
            assert False, "language '" + lang + "' advertised but non-existent"


def test_provs_are_available(broker):
    """Test whether all advertised providers are in fact available."""
    for lang, prov in broker.list_dicts():
        assert broker.dict_exists(lang)
        if not broker.dict_exists(lang):
            assert False, "language '" + lang + "' advertised but non-existent"
        if prov not in broker.describe():
            assert False, "provier '" + str(prov) + "' advertised but non-existent"


def test_prov_ordering(broker):
    """Test that provider ordering works correctly."""
    langs = {}
    provs = []
    # Find the providers for each language, and a list of all providers
    for tag, prov in broker.list_dicts():
        # Skip hyphenation dictionaries installed by OOo
        if tag.startswith("hyph_") and prov.name == "myspell":
            continue
        # Canonicalize separators
        tag = tag.replace("-", "_")
        langs[tag] = []
        # NOTE: we are excluding Zemberek here as it appears to return
        #       a broker for any language, even nonexistent ones
        if prov not in provs and prov.name != "zemberek":
            provs.append(prov)
    for prov in provs:
        for tag in langs:
            b2 = Broker()
            b2.set_ordering(tag, prov.name)
            try:
                d = b2.request_dict(tag)
                if d.provider != prov:
                    raise ValueError()
                langs[tag].append(prov)
            # TODO: bare except
            except:  # noqa
                pass
    # Check availability using a single entry in ordering
    for tag in langs:
        for prov in langs[tag]:
            b2 = Broker()
            b2.set_ordering(tag, prov.name)
            d = b2.request_dict(tag)
            assert (d.provider, tag) == (prov, tag)
            del d
            del b2
    # Place providers that don't have the language in the ordering
    for tag in langs:
        for prov in langs[tag]:
            order = prov.name
            for prov2 in provs:
                if prov2 not in langs[tag]:
                    order = prov2.name + "," + order
            b2 = Broker()
            b2.set_ordering(tag, order)
            d = b2.request_dict(tag)
            assert (d.provider, tag, order) == (prov, tag, order)
            del d
            del b2


def test_get_set_param(broker):
    """
    Scenario:
      Either broker.set_param(key, value) works
      Or broker.set_param(key, value) throws
      AttributeError

    """
    key = "pyenchant.unittest"
    value = "testing"

    error = None
    try:
        broker.set_param(key, value)
    except AttributeError as e:
        error = e

    if error:
        return
    else:
        assert broker.get_param("pyenchant.unittest") == "testing"
        other_broker = Broker()
        assert other_broker.get_param("pyenchant.unittest") is None



# pyenchant
#
# Copyright (C) 2004-2008, Ryan Kelly
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
#
# In addition, as a special exception, you are
# given permission to link the code of this program with
# non-LGPL Spelling Provider libraries (eg: a MSFT Office
# spell checker backend) and distribute linked combinations including
# the two.  You must obey the GNU Lesser General Public License in all
# respects for all of the code used other than said providers.  If you modify
# this file, you may extend this exception to your version of the
# file, but you are not obligated to do so.  If you do not wish to
# do so, delete this exception statement from your version.
#
import textwrap

import pytest

from enchant.tokenize import (
    EmailFilter,
    HTMLChunker,
    URLFilter,
    WikiWordFilter,
    basic_tokenize,
    empty_tokenize,
    get_tokenizer,
    wrap_tokenizer,
)
from enchant.tokenize.en import tokenize as tokenize_en


def test_basic_tokenize():
    """Simple regression test for basic white-space tokenization."""
    input = """This is a paragraph.  It's not very special, but it's designed
2 show how the splitter works with many-different combos
of words. Also need to "test" the (handling) of 'quoted' words."""
    assert list(basic_tokenize(input)) == [
        ("This", 0),
        ("is", 5),
        ("a", 8),
        ("paragraph", 10),
        ("It's", 22),
        ("not", 27),
        ("very", 31),
        ("special", 36),
        ("but", 45),
        ("it's", 49),
        ("designed", 54),
        ("2", 63),
        ("show", 65),
        ("how", 70),
        ("the", 74),
        ("splitter", 78),
        ("works", 87),
        ("with", 93),
        ("many-different", 98),
        ("combos", 113),
        ("of", 120),
        ("words", 123),
        ("Also", 130),
        ("need", 135),
        ("to", 140),
        ("test", 144),
        ("the", 150),
        ("handling", 155),
        ("of", 165),
        ("quoted", 169),
        ("words", 177),
    ]


def test_tokenize_strip():
    """Test special-char-stripping edge-cases in basic_tokenize."""
    input = "((' <this> \"\" 'text' has (lots) of (special chars} >>]"
    assert list(basic_tokenize(input)) == [
        ("<this>", 4),
        ("text", 15),
        ("has", 21),
        ("lots", 26),
        ("of", 32),
        ("special", 36),
        ("chars}", 44),
        (">>", 51),
    ]


def test_wrap_tokenizer():
    """Test wrapping of one tokenizer with another."""
    input = "this-string will be split@according to diff'rnt rules"
    from enchant.tokenize import en

    tknzr = wrap_tokenizer(basic_tokenize, en.tokenize)
    tknzr = tknzr(input)
    assert tknzr._tokenizer.__class__ == basic_tokenize
    assert tknzr._tokenizer.offset == 0
    for n, (word, pos) in enumerate(tknzr):
        if n == 0:
            assert pos == 0
            assert word == "this"
        if n == 1:
            assert pos == 5
            assert word == "string"
        if n == 2:
            assert pos == 12
            assert word == "will"
            # Test setting offset to a previous token
            tknzr.set_offset(5)
            assert tknzr.offset == 5
            assert tknzr._tokenizer.offset == 5
            assert tknzr._curtok.__class__ == empty_tokenize
        if n == 3:
            assert word == "string"
            assert pos == 5
        if n == 4:
            assert pos == 12
            assert word == "will"
        if n == 5:
            assert pos == 17
            assert word == "be"
            # Test setting offset past the current token
            tknzr.set_offset(20)
            assert tknzr.offset == 20
            assert tknzr._tokenizer.offset == 20
            assert tknzr._curtok.__class__ == empty_tokenize
        if n == 6:
            assert pos == 20
            assert word == "split"
        if n == 7:
            assert pos == 26
            assert word == "according"
            # Test setting offset to middle of current token
            tknzr.set_offset(23)
            assert tknzr.offset == 23
            assert tknzr._tokenizer.offset == 23
        if n == 8:
            assert pos == 23
            assert word == "it"
        # OK, I'm pretty happy with the behaviour, no need to
        # continue testing the rest of the string


@pytest.fixture
def test_text():
    text = """this text with http://url.com and SomeLinksLike
              ftp://my.site.com.au/some/file AndOthers not:/quite.a.url
              with-an@aemail.address as well"""
    return text


def test_url_filter(test_text):
    """Test filtering of URLs"""
    tknzr = get_tokenizer("en_US", filters=(URLFilter,))
    assert list(tknzr(test_text)) == [
        ("this", 0),
        ("text", 5),
        ("with", 10),
        ("and", 30),
        ("SomeLinksLike", 34),
        ("AndOthers", 93),
        ("not", 103),
        ("quite", 108),
        ("a", 114),
        ("url", 116),
        ("with", 134),
        ("an", 139),
        ("aemail", 142),
        ("address", 149),
        ("as", 157),
        ("well", 160),
    ]


def test_wiki_word_filter(test_text):
    """Test filtering of WikiWords"""
    tknzr = get_tokenizer("en_US", filters=(WikiWordFilter,))
    assert list(tknzr(test_text)) == [
        ("this", 0),
        ("text", 5),
        ("with", 10),
        ("http", 15),
        ("url", 22),
        ("com", 26),
        ("and", 30),
        ("ftp", 62),
        ("my", 68),
        ("site", 71),
        ("com", 76),
        ("au", 80),
        ("some", 83),
        ("file", 88),
        ("not", 103),
        ("quite", 108),
        ("a", 114),
        ("url", 116),
        ("with", 134),
        ("an", 139),
        ("aemail", 142),
        ("address", 149),
        ("as", 157),
        ("well", 160),
    ]


def test_email_filter(test_text):
    """Test filtering of email addresses"""
    tknzr = get_tokenizer("en_US", filters=(EmailFilter,))
    assert list(tknzr(test_text)) == [
        ("this", 0),
        ("text", 5),
        ("with", 10),
        ("http", 15),
        ("url", 22),
        ("com", 26),
        ("and", 30),
        ("SomeLinksLike", 34),
        ("ftp", 62),
        ("my", 68),
        ("site", 71),
        ("com", 76),
        ("au", 80),
        ("some", 83),
        ("file", 88),
        ("AndOthers", 93),
        ("not", 103),
        ("quite", 108),
        ("a", 114),
        ("url", 116),
        ("as", 157),
        ("well", 160),
    ]


def test_combined_filter(test_text):
    """Test several filters combined"""
    tknzr = get_tokenizer("en_US", filters=(URLFilter, WikiWordFilter, EmailFilter))
    assert list(tknzr(test_text)) == [
        ("this", 0),
        ("text", 5),
        ("with", 10),
        ("and", 30),
        ("not", 103),
        ("quite", 108),
        ("a", 114),
        ("url", 116),
        ("as", 157),
        ("well", 160),
    ]


def test_html_chunker():
    """Test filtering of URLs"""
    text = """hello<html><head><title>my title</title></head><body>this is a
              <b>simple</b> HTML document for <p> test<i>ing</i> purposes</p>.
            It < contains > various <-- special characters.
            """
    tknzr = get_tokenizer("en_US", chunkers=(HTMLChunker,))
    assert list(tknzr(text)) == [
        ("hello", 0),
        ("my", 24),
        ("title", 27),
        ("this", 53),
        ("is", 58),
        ("a", 61),
        ("simple", 80),
        ("HTML", 91),
        ("document", 96),
        ("for", 105),
        ("test", 113),
        ("ing", 120),
        ("purposes", 128),
        ("It", 154),
        ("contains", 159),
        ("various", 170),
        ("special", 182),
        ("characters", 190),
    ]


def test_tokenize_en():
    """Simple regression test for English tokenization."""
    input = """This is a paragraph.  It's not very special, but it's designed
2 show how the splitter works with many-different combos
of words. Also need to "test" the handling of 'quoted' words."""
    assert list(tokenize_en(input)) == [
        ("This", 0),
        ("is", 5),
        ("a", 8),
        ("paragraph", 10),
        ("It's", 22),
        ("not", 27),
        ("very", 31),
        ("special", 36),
        ("but", 45),
        ("it's", 49),
        ("designed", 54),
        ("show", 65),
        ("how", 70),
        ("the", 74),
        ("splitter", 78),
        ("works", 87),
        ("with", 93),
        ("many", 98),
        ("different", 103),
        ("combos", 113),
        ("of", 120),
        ("words", 123),
        ("Also", 130),
        ("need", 135),
        ("to", 140),
        ("test", 144),
        ("the", 150),
        ("handling", 154),
        ("of", 163),
        ("quoted", 167),
        ("words", 175),
    ]


def test_unicode_basic():
    """Test tokenization of a basic unicode string."""
    input = "Ik ben geïnteresseerd in de coördinatie van mijn knieën, maar kan niet één à twee enquêtes vinden die recht doet aan mijn carrière op Curaçao"
    output = input.split(" ")
    output[8] = output[8][0:-1]
    for itm_o, itm_v in zip(output, tokenize_en(input)):
        assert itm_o == itm_v[0]
        assert input[itm_v[1] :].startswith(itm_o)


def test_bug1591450():
    """Check for tokenization regressions identified in bug #1591450."""
    input = """Testing <i>markup</i> and {y:i}so-forth...leading dots and trail--- well, you get-the-point. Also check numbers: 999 1,000 12:00 .45. Done?"""
    assert list(tokenize_en(input)) == [
        ("Testing", 0),
        ("i", 9),
        ("markup", 11),
        ("i", 19),
        ("and", 22),
        ("y", 27),
        ("i", 29),
        ("so", 31),
        ("forth", 34),
        ("leading", 42),
        ("dots", 50),
        ("and", 55),
        ("trail", 59),
        ("well", 68),
        ("you", 74),
        ("get", 78),
        ("the", 82),
        ("point", 86),
        ("Also", 93),
        ("check", 98),
        ("numbers", 104),
        ("Done", 134),
    ]


def test_bug2785373():
    """Testcases for bug #2785373"""
    input = "So, one dey when I wes 17, I left."
    for _ in tokenize_en(input):
        pass
    input = "So, one dey when I wes 17, I left."
    for _ in tokenize_en(input):
        pass


def test_finnish_text():
    """Test tokenizing some Finnish text.

    This really should work since there are no special rules to apply,
    just lots of non-ascii characters.
    """
    text = textwrap.dedent(
        """\
        Tämä on kappale. Eipä ole kovin 2 nen, mutta tarkoitus on näyttää miten sanastaja
         toimii useiden-erilaisten sanaryppäiden kimpussa.
        Pitääpä vielä 'tarkistaa' sanat jotka "lainausmerkeissä". Heittomerkki ja vaa'an.
        Ulkomaisia sanoja süss, spaß.
    """
    )
    assert list(tokenize_en(text)) == [
        ("Tämä", 0),
        ("on", 5),
        ("kappale", 8),
        ("Eipä", 17),
        ("ole", 22),
        ("kovin", 26),
        ("nen", 34),
        ("mutta", 39),
        ("tarkoitus", 45),
        ("on", 55),
        ("näyttää", 58),
        ("miten", 66),
        ("sanastaja", 72),
        ("toimii", 83),
        ("useiden", 90),
        ("erilaisten", 98),
        ("sanaryppäiden", 109),
        ("kimpussa", 123),
        ("Pitääpä", 133),
        ("vielä", 141),
        ("tarkistaa", 148),
        ("sanat", 159),
        ("jotka", 165),
        ("lainausmerkeissä", 172),
        ("Heittomerkki", 191),
        ("ja", 204),
        ("vaa'an", 207),
        ("Ulkomaisia", 215),
        ("sanoja", 226),
        ("süss", 233),
        ("spaß", 239),
    ]


def test_typographic_apostrophe():
    """ "Typographic apostrophes should be word separators in English."""
    text = "They\u2019re here"
    assert list(tokenize_en(text)) == [
        ("They", 0),
        ("re", 5),
        ("here", 8),
    ]


@pytest.mark.parametrize(
    "text,expected",
    [
        ("", []),
        (b"", []),
        (bytearray(), []),
        ("a", [("a", 0)]),
        (b"a", [(b"a", 0)]),
        (bytearray((97,)), [(bytearray((97,)), 0)]),
        ("ä", [("ä", 0)]),
        (b"\xc3", []),
        (b"\xc3\xa4", [(b"\xc3\xa4", 0)]),
        (bytearray((0xC3,)), []),
        (bytearray((0xC3, 0xA4)), [(bytearray((0xC3, 0xA4)), 0)]),
    ],
)
def test_tokenize_en_byte(text, expected):
    """Test tokenizing bytes."""
    assert list(tokenize_en(text)) == expected



import sys

import pytest

from enchant import DictWithPWL, PyPWL, request_pwl_dict


@pytest.fixture
def pwl_path(tmp_path):
    res = tmp_path / ("pwl.txt")
    res.write_text("")
    return res


def set_pwl_contents(path, contents):
    """Set the contents of the PWL file."""
    path.write_text("\n".join(contents))


def get_pwl_contents(path):
    """Retrieve the contents of the PWL file."""
    contents = path.read_text()
    return [c.strip() for c in contents.splitlines()]


def test_check(pwl_path):
    """Test that basic checking works for PWLs."""
    set_pwl_contents(pwl_path, ["Sazz", "Lozz"])
    d = request_pwl_dict(str(pwl_path))
    assert d.check("Sazz")
    assert d.check("Lozz")
    assert not d.check("hello")


@pytest.mark.skipif(
    sys.implementation.name == "pypy" and sys.platform == "win32",
    reason="failing for an unknown reason",
)
# This test only fails on mypy3 and Windows. Not sure if it's
# a bug in PyEnchant, Enchant or pypy3
def test_unicodefn(tmp_path):
    """Test that unicode PWL filenames are accepted."""
    unicode_path = tmp_path / "테스트"
    set_pwl_contents(unicode_path, ["Lozz"])
    d = request_pwl_dict(str(unicode_path))
    assert d.check("Lozz")
    assert d


def test_add(pwl_path):
    """Test that adding words to a PWL works correctly."""
    d = request_pwl_dict(str(pwl_path))
    assert not d.check("Flagen")
    d.add("Esquilax")
    d.add("Esquilam")
    assert d.check("Esquilax")
    assert "Esquilax" in get_pwl_contents(pwl_path)
    assert d.is_added("Esquilax")


def test_dwpwl(tmp_path, pwl_path):
    """Test functionality of DictWithPWL."""
    set_pwl_contents(pwl_path, ["Sazz", "Lozz"])
    other_path = tmp_path / "pel.txt"
    d = DictWithPWL("en_US", str(pwl_path), str(other_path))
    assert d.check("Sazz")
    assert d.check("Lozz")
    assert d.check("hello")
    assert not d.check("helo")
    assert not d.check("Flagen")
    d.add("Flagen")
    assert d.check("Flagen")
    assert "Flagen" in get_pwl_contents(pwl_path)
    d.remove("hello")
    assert not d.check("hello")
    assert "hello" not in d.suggest("helo")
    d.remove("Lozz")
    assert not d.check("Lozz")


def test_dwpwl_empty(tmp_path):
    """Test functionality of DictWithPWL using transient dicts."""
    d = DictWithPWL("en_US", None, None)
    assert d.check("hello")
    assert not d.check("helo")
    assert not d.check("Flagen")
    d.add("Flagen")
    assert d.check("Flagen")
    d.remove("hello")
    assert not d.check("hello")
    d.add("hello")
    assert d.check("hello")


def test_pypwl(tmp_path):
    """Test our pure-python PWL implementation."""
    d = PyPWL()
    assert list(d._words) == []
    d.add("hello")
    d.add("there")
    d.add("duck")
    ws = list(d._words)
    assert len(ws) == 3
    assert "hello" in ws
    assert "there" in ws
    assert "duck" in ws
    d.remove("duck")
    d.remove("notinthere")
    ws = list(d._words)
    assert len(ws) == 2
    assert "hello" in ws
    assert "there" in ws
